#
# generate_core_keys.py
#
# Author: Juan A. Suarez Romero <jasuarez@igalia.com>
#
# Copyright (C) 2016 Igalia S.L.
#
# Generates a .c file containing an array with the core keys
#

import re
import sys

if len(sys.argv) != 3:
    print("Usage: " + sys.argv[0] + " <input> <output>")
    exit(1)

finput = open(sys.argv[1], "r")
foutput = open(sys.argv[2], "w")

foutput.write("gchar *grl_core_keys[] = {\n")

output_keys = False
for line in finput:
    if re.search("BEGIN CORE KEYS", line):
        output_keys = True
    if re.search("END CORE KEYS", line):
        output_keys = False
    m = re.search("GRL_METADATA_KEY_[^ ]+", line)
    if output_keys and m:
            foutput.write("\"" +  m.group(0) + "\",\n")

foutput.write("};");

finput.close()
foutput.close()



import unittest
import logging

try:
    from gi.repository import Grl
except:
    logging.warning("Unable to import Grilo's introspection bindings")
    exit()
try:
    from gi.repository import GLib
except TypeError:
    # there seems to be a bug with GVariants these days that triggers an
    # exception when importing GLib. Seems to be harmless though. We import
    # explicitly GLib in a try/except here so that we can handle that, else the
    # exception would be raised the first time it is imported by other code
    # (e.g. by using a GDateTime)
    from gi.repository import GLib

class TestMedia(unittest.TestCase):
    def test_creation(self):
        media = Grl.Media.new()
        self.assertFalse(media is None)

    def test_publication_date(self):
        media = Grl.Media.new()
        date_string = "2001-02-03T04:05:06Z"
        date = Grl.date_time_from_iso8601 (date_string)

        self.assertTrue(media.get_publication_date() is None)
        media.set_publication_date(date)
        d = media.get_publication_date()
        self.assertTrue(d is not None)
        self.assertEqual(d.get_year(), 2001)
        self.assertEqual(d.get_month(), 2)
        self.assertEqual(d.get_day_of_month(), 3)
        self.assertEqual(d.get_hour(), 4)
        self.assertEqual(d.get_minute(), 5)
        self.assertEqual(d.get_second(), 6)
        self.assertEqual(d.format("%FT%H:%M:%S%Z"), date.format("%FT%H:%M:%S%Z"))

        # date only
        date_string = "2001-02-03"
        date = Grl.date_time_from_iso8601 (date_string)
        media.set_publication_date(date)
        d = media.get_publication_date()
        self.assertTrue(d is not None)
        self.assertEqual(d.get_year(), 2001)
        self.assertEqual(d.get_month(), 2)
        self.assertEqual(d.get_day_of_month(), 3)
        self.assertEqual(d.get_hour(), 12)
        self.assertEqual(d.get_minute(), 0)
        self.assertEqual(d.get_second(), 0)
        self.assertEqual(d.format("%FT%H:%M:%S%Z"), date.format("%FT%H:%M:%S%Z"))
        self.assertEqual(d.format("%FT%H:%M:%S%Z"), date_string + "T12:00:00UTC")

# who said "this is ugly" ?
Grl.init([])
registry = Grl.Registry.get_default()
try:
    registry.load_plugin_by_id("grl-filesystem")
except GLib.Error:
    pass



#!/usr/bin/env python

import unittest
import glob
import sys

# https://bugzilla.gnome.org/show_bug.cgi?id=748455
exit (0)

test_loader = unittest.defaultTestLoader

names = []
args = sys.argv[1:]

if args:
    for item in args:
        names.append(item[:-3])
else:
    for filename in glob.iglob("test_*.py"):
        names.append(filename[:-3])

test_suites = []
for name in names:
    test_suites.append(test_loader.loadTestsFromName(name))

runner = unittest.TextTestRunner(verbosity=2)

for suite in test_suites:
    runner.run(suite)



import unittest
import logging

try:
    from gi.repository import Grl
except:
    logging.warning("Unable to import Grilo's introspection bindings")
    exit()

class TestMediaPlugin(unittest.TestCase):

    NONEXISTING_KEY = 'nonexisting_key'

    def __init__(self, method_name):
        super(TestMediaPlugin, self).__init__(method_name)
        Grl.init([])
        self.registry = Grl.Registry.get_default()
        self.registry.load_all_plugins()
        sources = self.registry.get_sources(False)
        if sources:
            self.plugin = sources[0]

    def test_get_name(self):
        name = self.plugin.get_name()
        self.assertTrue(name)

    def test_get_description(self):
        description = self.plugin.get_name()
        self.assertTrue(description)

    def test_get_version(self):
        version = self.plugin.get_version()
        self.assertTrue(version)

    def test_get_license(self):
        license = self.plugin.get_license()
        self.assertTrue(license)

    def test_get_author(self):
        author = self.plugin.get_author()
        self.assertTrue(author)

    def test_get_site(self):
        site = self.plugin.get_site()
        self.assertTrue(site)

    def test_get_id(self):
        id = self.plugin.get_id()
        self.assertTrue(id)

    def test_get_filename(self):
        filename = self.plugin.get_filename()
        self.assertTrue(filename)

    def test_get_rank(self):
        rank = self.plugin.get_rank()
        self.assertTrue(rank is not None)

    def test_get_info_keys(self):
        info_keys = self.plugin.get_info_keys()
        self.assertTrue(info_keys)

    def test_get_info_existing_key(self):
        keys = self.plugin.get_info_keys()
        if keys:
            existing_key = keys[0]
            info = self.plugin.get_info(existing_key)
            self.assertTrue(info)

    def test_get_info_nonexisting_key(self):
        info = self.plugin.get_info(self.NONEXISTING_KEY)
        self.assertFalse(info)



# some tests on filesystem, useful to test core stuff as long as we don't have
# proper mock sources.
import unittest
import os, tempfile, shutil, time, calendar

try:
    from gi.repository import Grl
except:
    logging.warning("Unable to import Grilo's introspection bindings")
    exit()

try:
    from gi.repository import GLib
except TypeError:
    # there seems to be a bug with GVariants these days that triggers an
    # exception when importing GLib. Seems to be harmless though. We import
    # explicitly GLib in a try/except here so that we can handle that, else the
    # exception would be raised the first time it is imported by other code
    # (e.g. by using a GDateTime)
    from gi.repository import GLib



_tempdir = tempfile.mkdtemp()


class TestFSOperations(unittest.TestCase):
    def __init__(self, method_name):
        super(TestFSOperations, self).__init__(method_name)
        self.registry = Grl.Registry.get_default()
        self.plugin = self.registry.lookup_source("grl-filesystem")

        self.file_tree_pictures = [
            "a/aa/a.jpg",
            "a/ab/a.jpg",
            "x/yz/a.png",
            "a/a.bmp",
        ]
        self.file_tree_audio = [
            "a/aa/a.mp3",
            "a/ab/a.mp3",
            "a/a.ogg",
        ]
        self.file_tree_video = [
            "a/aa/aa7.avi",
            "x/yz/aa8.mpeg",
            "a/ab/aa9.mkv",
        ]

        self.file_tree = self.file_tree_pictures \
                         + self.file_tree_audio \
                         + self.file_tree_video



    def _touch_file(self, path):
        dirname = os.path.dirname(path)
        if not os.path.isdir(dirname):
            os.makedirs(dirname)
        open(path, 'w')

    def setUp(self):
        file_tree =  (os.path.join(_tempdir, d) for d in self.file_tree)

        for f in file_tree:
            self._touch_file(f)

        self._old_files = [os.path.join(_tempdir, f) \
                          for f in (self.file_tree_pictures[1],
                                    self.file_tree_audio[1],
                                    self.file_tree_video[1])]
        for f in self._old_files:
            mtime = calendar.timegm(time.strptime("1989-07-14", "%Y-%m-%d"))
            os.utime(f, (mtime, mtime))

    def tearDown(self):
        if _tempdir and os.path.isdir(_tempdir):
            shutil.rmtree(_tempdir)

    def test_search(self):
        caps = self.plugin.get_caps(Grl.SupportedOps.SEARCH)
        options = Grl.OperationOptions.new(caps)
        l = self.plugin.search_sync("a", [Grl.METADATA_KEY_ID], options)
        self.assertNotEqual(l, None)
        expected_result = sorted(os.path.join(_tempdir, d) for d in self.file_tree)
        path_list = [m.get_id() for m in l]
        path_list.sort()
        self.assertEqual(path_list, expected_result)

    def test_type_filtered_search(self):
        caps = self.plugin.get_caps(Grl.SupportedOps.SEARCH)
        options = Grl.OperationOptions.new(caps)
        options.set_type_filter(Grl.TypeFilter.VIDEO)
        l = self.plugin.search_sync("a", [Grl.METADATA_KEY_ID], options)
        self.assertNotEqual(l, None)
        expected_result = sorted(os.path.join(_tempdir, d) for d in self.file_tree_video)
        path_list = sorted(m.get_id() for m in l)
        self.assertEqual(path_list, expected_result)

    def test_key_filtered_search(self):
        caps = self.plugin.get_caps(Grl.SupportedOps.SEARCH)
        options = Grl.OperationOptions.new(caps)
        options.set_key_filters({Grl.METADATA_KEY_MIME: 'image/jpeg'})
        l = self.plugin.search_sync("a", [Grl.METADATA_KEY_ID, Grl.METADATA_KEY_MIME], options)
        expected_result = sorted(os.path.join(_tempdir, d)
                                 for d in self.file_tree_pictures
                                 if d.endswith('.jpg'))
        path_list = sorted(m.get_id() for m in l)
        self.assertEqual(path_list, expected_result)

    def test_key_range_filtered_search(self):
        caps = self.plugin.get_caps(Grl.SupportedOps.SEARCH)
        options = Grl.OperationOptions.new(caps)
        # For now, we provide date _and_ time because of
        # https://bugzilla.gnome.org/show_bug.cgi?id=650968
        start = Grl.date_time_from_iso8601("1989-07-13T00:00:00Z")
        end = Grl.date_time_from_iso8601("1989-07-15T00:00:00Z")
        start.ref()
        end.ref()
        options.set_key_range_filter_value(Grl.METADATA_KEY_MODIFICATION_DATE,
                                           start, end)
        l = self.plugin.search_sync("a",
                                    [Grl.METADATA_KEY_ID,
                                    Grl.METADATA_KEY_MODIFICATION_DATE],
                                    options)
        expected_result = sorted(self._old_files)
        path_list = sorted(m.get_id() for m in l)
        self.assertEqual(path_list, expected_result)


    def test_browse(self):
        caps = self.plugin.get_caps(Grl.SupportedOps.SEARCH)
        options = Grl.OperationOptions.new(caps)
        #self._results = []
        results = []
        def recursive_browse(box, results=results):
            new_results = self.plugin.browse_sync(box, [Grl.METADATA_KEY_ID], options)
            results += new_results
            for m in new_results:
                if isinstance(m, Grl.MediaBox):
                    recursive_browse(m)

        recursive_browse(None)
        path_list = sorted(m.get_id() for m in results if not isinstance(m, Grl.MediaBox))
        expected_result = sorted(os.path.join(_tempdir, d) for d in self.file_tree)
        self.assertEqual(path_list, expected_result)

    def test_search_async(self):
        l = []
        def callback(source, operation_id, media, remaining, user_data, error):
            try:
                if media:
                    l.append(media)
                if remaining == 0:
                    self.assertNotEqual(l, None)
                    expected_result = sorted(os.path.join(_tempdir, d) for d in self.file_tree)
                    path_list = [m.get_id() for m in l]
                    path_list.sort()
                    self.assertEqual(path_list, expected_result)
            finally:
                if remaining == 0:
                    loop.quit()

        caps = self.plugin.get_caps(Grl.SupportedOps.SEARCH)
        options = Grl.OperationOptions.new(caps)
        self.plugin.search("a", [Grl.METADATA_KEY_ID], options, callback, None)

        loop = GLib.MainLoop()
        loop.run()

    def test_type_filtered_search_async(self):
        l = []
        caps = self.plugin.get_caps(Grl.SupportedOps.SEARCH)
        options = Grl.OperationOptions.new(caps)
        options.set_type_filter(Grl.TypeFilter.VIDEO)

        def callback(source, operation_id, media, remaining, user_data, error):
            try:
                if media:
                    l.append(media)
                if remaining == 0:
                    self.assertNotEqual(l, None)
                    expected_result = sorted(os.path.join(_tempdir, d) for d in self.file_tree_video)
                    path_list = sorted(m.get_id() for m in l)
                    self.assertEqual(path_list, expected_result)
            finally:
                if remaining == 0:
                    loop.quit()


        self.plugin.search("a", [Grl.METADATA_KEY_ID], options, callback, None)

        loop = GLib.MainLoop()
        loop.run()

    def test_key_filtered_search_async(self):
        l = []
        def callback(source, operation_id, media, remaining, user_data, error):
            try:
                if media:
                    l.append(media)
                if remaining == 0:
                    expected_result = sorted(os.path.join(_tempdir, d)
                                             for d in self.file_tree_pictures
                                             if d.endswith('.jpg'))
                    path_list = sorted(m.get_id() for m in l)
                    self.assertEqual(path_list, expected_result)
            finally:
                if remaining == 0:
                    loop.quit()

        caps = self.plugin.get_caps(Grl.SupportedOps.SEARCH)
        options = Grl.OperationOptions.new(caps)
        options.set_key_filters({Grl.METADATA_KEY_MIME: 'image/jpeg'})
        self.plugin.search("a", [Grl.METADATA_KEY_ID, Grl.METADATA_KEY_MIME], options, callback, None)

        loop = GLib.MainLoop()
        loop.run()

    def test_browse_async(self):
        caps = self.plugin.get_caps(Grl.SupportedOps.SEARCH)
        options = Grl.OperationOptions.new(caps)
        l = []
        results = []
        def callback(source, operation_id, media, remaining, user_data, error):
            if media:
                l.append(media)
            if remaining == 0:
                # todo: recurse
                dirs = []
                for m in l:
                    if isinstance(m, Grl.MediaBox):
                        dirs.append(m)
                    else:
                        results.append(m)
                l[:] = []
                for m in dirs:
                    self.plugin.browse(m, [Grl.METADATA_KEY_ID], options, callback, None)


        def check_result():
            loop.quit()
            path_list = sorted(m.get_id() for m in results)
            expected_result = sorted(os.path.join(_tempdir, d) for d in self.file_tree)
            self.assertEqual(path_list, expected_result)

            return False

        self.plugin.browse(None, [Grl.METADATA_KEY_ID], options, callback, None)

        loop = GLib.MainLoop()
        GLib.timeout_add(1000, check_result)
        loop.run()


def _init():
    Grl.init([])
    registry = Grl.Registry.get_default()

    fs_config = Grl.Config.new("grl-filesystem", None)
    fs_config.set_string("base-path", _tempdir)
    registry.add_config(fs_config)

    try:
        registry.load_plugin_by_id("grl-filesystem")
    except GLib.Error:
        pass

_init()



KEY_NONEXISTING = 'nonexisting-key'
KEY_ALBUM = 'album'
KEY_ARTIST = 'artist'
KEY_AUTHOR = 'author'
KEY_BITRATE = 'bitrate'
KEY_CERTIFICATE = 'certificate'
KEY_CHILDCOUNT = 'childcount'
KEY_DATE = 'date'
KEY_DESCRIPTION = 'description'
KEY_DURATION = 'duration'
KEY_EXTERNAL_PLAYER = 'external-player'
KEY_EXTERNAL_URL = 'external-url'
KEY_FRAMERATE = 'framerate'
KEY_GENRE = 'genre'
KEY_HEIGHT = 'height'
KEY_ID = 'id'
KEY_LAST_PLAYED = 'last-played'
KEY_LAST_POSITION = 'last-position'
KEY_LICENSE = 'license'
KEY_LYRICS = 'lyrics'
KEY_MIME = 'mime'
KEY_PLAY_COUNT = 'play-count'
KEY_RATING = 'rating'
KEY_SITE = 'site'
KEY_SOURCE = 'source'
KEY_STUDIO = 'studio'
KEY_THUMBNAIL = 'thumbnail'
KEY_TITLE = 'title'
KEY_URL = 'url'
KEY_WIDTH = 'width'

REGISTERED_KEYS = [KEY_ALBUM, KEY_ARTIST, KEY_AUTHOR, KEY_BITRATE,
                   KEY_CERTIFICATE, KEY_CHILDCOUNT, KEY_DATE,
                   KEY_DESCRIPTION, KEY_DURATION, KEY_EXTERNAL_PLAYER,
                   KEY_EXTERNAL_URL, KEY_FRAMERATE, KEY_GENRE, KEY_HEIGHT,
                   KEY_ID, KEY_LAST_PLAYED, KEY_LAST_POSITION, KEY_LICENSE,
                   KEY_LYRICS, KEY_MIME, KEY_PLAY_COUNT, KEY_RATING,
                   KEY_SITE, KEY_SOURCE, KEY_STUDIO, KEY_THUMBNAIL,
                   KEY_TITLE, KEY_URL, KEY_WIDTH]

SUPPORTED_OPS = []
try:
    from gi.repository import Grl
    SUPPORTED_OPS = [Grl.SupportedOps.NONE, Grl.SupportedOps.METADATA,
                     Grl.SupportedOps.RESOLVE, Grl.SupportedOps.BROWSE,
                     Grl.SupportedOps.SEARCH, Grl.SupportedOps.QUERY,
                     Grl.SupportedOps.STORE, Grl.SupportedOps.STORE_PARENT,
                     Grl.SupportedOps.REMOVE, Grl.SupportedOps.SET_METADATA]
except:
    pass



import unittest
import logging

try:
    from gi.repository import Grl
except:
    logging.warning("Unable to import Grilo's introspection bindings")
    exit()
try:
    from gi.repository import GLib
except TypeError:
    # there seems to be a bug with GVariants these days that triggers an
    # exception when importing GLib. Seems to be harmless though. We import
    # explicitly GLib in a try/except here so that we can handle that, else the
    # exception would be raised the first time it is imported by other code
    # (e.g. by using a GDateTime)
    from gi.repository import GLib

OPTION_TYPE_FILTER = "type-filter"

class TestCaps(unittest.TestCase):
    def test_creation(self):
        caps = Grl.Caps()
        self.assertFalse(caps is None)

    def test_mandatory_options(self):
        return None
        caps = Grl.Caps()
        for key in ("skip", "count", "flags"):
            self.assertTrue(caps.test_option(key, 0),
                            "test_option() returned False for %s" % key)

    def test_type_filter(self):
        caps = Grl.Caps()
        # test default value
        self.assertEqual(caps.get_type_filter(), Grl.TypeFilter.NONE)

        self.assertTrue(caps.test_option(OPTION_TYPE_FILTER, Grl.TypeFilter.NONE))
        self.assertFalse(caps.test_option(OPTION_TYPE_FILTER, Grl.TypeFilter.AUDIO))

        caps.set_type_filter(Grl.TypeFilter.VIDEO)
        self.assertEqual(caps.get_type_filter(), Grl.TypeFilter.VIDEO)
        self.assertTrue(caps.test_option(OPTION_TYPE_FILTER, Grl.TypeFilter.NONE))
        self.assertTrue(caps.test_option(OPTION_TYPE_FILTER, Grl.TypeFilter.VIDEO))
        self.assertFalse(caps.test_option(OPTION_TYPE_FILTER, Grl.TypeFilter.AUDIO))
        self.assertFalse(caps.test_option(OPTION_TYPE_FILTER,
                                          Grl.TypeFilter.AUDIO | Grl.TypeFilter.VIDEO))

    def test_key_filter(self):
        caps = Grl.Caps()
        self.assertEqual(caps.get_key_filter(), [])
        keys = [Grl.METADATA_KEY_ARTIST, Grl.METADATA_KEY_ALBUM]
        caps.set_key_filter(keys)
        self.assertEqual(caps.get_key_filter(), keys)
        self.assertTrue(caps.is_key_filter(Grl.METADATA_KEY_ARTIST))
        self.assertTrue(caps.is_key_filter(Grl.METADATA_KEY_ALBUM))
        self.assertFalse(caps.is_key_filter(Grl.METADATA_KEY_INVALID))
        self.assertFalse(caps.is_key_filter(Grl.METADATA_KEY_MODIFICATION_DATE))


class TestOptions(unittest.TestCase):
    def test_creation(self):
        caps = Grl.Caps()
        self.assertFalse(caps is None)
        options = Grl.OperationOptions.new(caps)
        self.assertFalse(options is None)
        self.assertFalse(Grl.OperationOptions() is None)

    def test_default_values(self):
        options = Grl.OperationOptions()
        self.assertEqual(options.get_skip(), 0)
        self.assertEqual(options.get_count(), Grl.COUNT_INFINITY)
        self.assertEqual(options.get_flags(),
                         Grl.MetadataResolutionFlags.NORMAL)
        self.assertEqual(options.get_type_filter(), Grl.TypeFilter.ALL)


    def test_value_setting_no_caps(self):
        options = Grl.OperationOptions()

        options.set_skip(12)
        self.assertEqual(options.get_skip(), 12)

        options.set_count(28)
        self.assertEqual(options.get_count(), 28)

        flags =Grl.MetadataResolutionFlags.FAST_ONLY \
                | Grl.MetadataResolutionFlags.IDLE_RELAY
        options.set_flags(flags)
        self.assertEqual(options.get_flags(), flags)

        fltr = Grl.TypeFilter.AUDIO | Grl.TypeFilter.VIDEO
        options.set_type_filter(fltr)
        self.assertEqual(options.get_type_filter(), fltr)

    def test_caps(self):
        caps = Grl.Caps()
        caps.set_type_filter(Grl.TypeFilter.VIDEO)
        options = Grl.OperationOptions.new(caps)
        self.assertTrue(options.set_type_filter(Grl.TypeFilter.VIDEO))
        self.assertFalse(options.set_type_filter(Grl.TypeFilter.AUDIO))
        self.assertFalse(options.set_type_filter(Grl.TypeFilter.IMAGE))

        # now with other caps, test obey_caps()
        caps2 = Grl.Caps()
        caps2.set_type_filter(Grl.TypeFilter.AUDIO)
        ret, supported, unsupported = options.obey_caps(caps2)
        self.assertFalse(ret)
        self.assertFalse(supported.key_is_set(Grl.OPERATION_OPTION_SKIP))
        self.assertFalse(supported.key_is_set(Grl.OPERATION_OPTION_COUNT))
        self.assertFalse(supported.key_is_set(Grl.OPERATION_OPTION_FLAGS))
        self.assertFalse(supported.key_is_set(OPTION_TYPE_FILTER))

        self.assertFalse(unsupported.key_is_set(Grl.OPERATION_OPTION_SKIP))
        self.assertFalse(unsupported.key_is_set(Grl.OPERATION_OPTION_COUNT))
        self.assertFalse(unsupported.key_is_set(Grl.OPERATION_OPTION_FLAGS))
        self.assertTrue(unsupported.key_is_set(OPTION_TYPE_FILTER))
        self.assertEqual(unsupported.get_type_filter(), Grl.TypeFilter.VIDEO)

        caps3 = Grl.Caps()
        caps3.set_type_filter(Grl.TypeFilter.VIDEO)
        ret, supported, unsupported = options.obey_caps(caps3)
        self.assertTrue(ret)
        self.assertFalse(supported.key_is_set(Grl.OPERATION_OPTION_SKIP))
        self.assertFalse(supported.key_is_set(Grl.OPERATION_OPTION_COUNT))
        self.assertFalse(supported.key_is_set(Grl.OPERATION_OPTION_FLAGS))
        self.assertTrue(supported.key_is_set(OPTION_TYPE_FILTER))
        self.assertEqual(supported.get_type_filter(), Grl.TypeFilter.VIDEO)

        self.assertFalse(unsupported.key_is_set(Grl.OPERATION_OPTION_SKIP))
        self.assertFalse(unsupported.key_is_set(Grl.OPERATION_OPTION_COUNT))
        self.assertFalse(unsupported.key_is_set(Grl.OPERATION_OPTION_FLAGS))
        self.assertFalse(unsupported.key_is_set(OPTION_TYPE_FILTER))

    def test_key_filters(self):
        options = Grl.OperationOptions.new(None)
        release_date = GLib.DateTime.new_utc(1994,1,1,12,0,0)
        self.assertEqual(options.get_key_filter_list(), [])
        self.assertTrue(options.set_key_filter_value(Grl.METADATA_KEY_ARTIST, "NOFX"))
        self.assertTrue(options.set_key_filters({
                                                Grl.METADATA_KEY_ALBUM: "Punk in Drublic",
                                                Grl.METADATA_KEY_CREATION_DATE: release_date}))
        filter_list = options.get_key_filter_list()
        filter_list.sort()
        expected_filters = [Grl.METADATA_KEY_ARTIST,
                            Grl.METADATA_KEY_ALBUM,
                            Grl.METADATA_KEY_CREATION_DATE]
        expected_filters.sort()
        self.assertEqual(filter_list, expected_filters)
        self.assertEqual(options.get_key_filter(Grl.METADATA_KEY_ARTIST), "NOFX")
        self.assertEqual(options.get_key_filter(Grl.METADATA_KEY_ALBUM), "Punk in Drublic")
        filter_date = options.get_key_filter(Grl.METADATA_KEY_CREATION_DATE)
        self.assertTrue(filter_date is not None)
        self.assertEqual(filter_date.format("%FT%H:%M:%S:%Z"), release_date.format("%FT%H:%M:%S:%Z"))

    def test_key_filters_with_caps(self):
        caps = Grl.Caps()
        release_date = GLib.DateTime.new_utc(1994,1,1,12,0,0)
        caps.set_key_filter([Grl.METADATA_KEY_ARTIST, Grl.METADATA_KEY_CREATION_DATE])
        options = Grl.OperationOptions.new(caps)
        self.assertTrue(options.set_key_filter_value(Grl.METADATA_KEY_ARTIST, "NOFX"))
        self.assertFalse(options.set_key_filter_value(Grl.METADATA_KEY_ALBUM, "Punk in Drublic"))
        self.assertTrue(options.set_key_filter_value(Grl.METADATA_KEY_CREATION_DATE, release_date))
        filter_list = options.get_key_filter_list()
        filter_list.sort()
        expected_filters = [Grl.METADATA_KEY_ARTIST,
                            Grl.METADATA_KEY_CREATION_DATE]
        expected_filters.sort()
        self.assertEqual(filter_list, expected_filters)
        self.assertEqual(options.get_key_filter(Grl.METADATA_KEY_ARTIST), "NOFX")
        self.assertEqual(options.get_key_filter(Grl.METADATA_KEY_ALBUM), None)

        filter_date = options.get_key_filter(Grl.METADATA_KEY_CREATION_DATE)
        self.assertTrue(filter_date is not None)
        self.assertEqual(filter_date.format("%FT%H:%M:%S:%Z"), release_date.format("%FT%H:%M:%S:%Z"))



class TestFileSystem(unittest.TestCase):
    def __init__(self, method_name):
        super(TestFileSystem, self).__init__(method_name)
        self.registry = Grl.Registry.get_default()
        self.plugin = self.registry.lookup_source("grl-filesystem")

    def test_caps(self):
        caps = self.plugin.get_caps(Grl.SupportedOps.BROWSE)
        self.assertFalse(caps is None)


# who said "this is ugly" ?
Grl.init([])
registry = Grl.Registry.get_default()
try:
    registry.load_plugin_by_id("grl-filesystem")
except GLib.Error:
    pass



import unittest
import os
import util
import tempfile
import logging
import constants
import glib

try:
    from gi.repository import Grl
except:
    logging.warning("Unable to import Grilo's introspection bindings")
    exit()

class TestRegistry(unittest.TestCase):

    EXISTING_LIBRARY_PATH = None

    NONEXISTING_LIBRARY_PATH = 'NONEXISTING_LIBRARY.so'

    INVALID_LIBRARY_PATH = os.path.join(util.PREFIX,
                                        util.GRL_LIB_NAME)

    NONEXISTING_SOURCE = 'NON_EXISTING_SOURCE'

    def __init__(self, method_name):
        super(TestRegistry, self).__init__(method_name)
        Grl.init([])
        self.registry = Grl.Registry.get_default()
        plugin_paths = util.GRL_PLUGIN_PATH.split(':')
        for path in plugin_paths:
            if path:
                entries = os.listdir(path)
                self.EXISTING_LIBRARY_PATH = os.path.join(path,
                                                          entries[0])
                break
        print self.EXISTING_LIBRARY_PATH

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def test_get_default_not_null(self):
        registry = Grl.Registry.get_default()
        self.assertTrue(registry)

    def test_get_default_singleton(self):
        registry1 = Grl.Registry.get_default()
        registry2 = Grl.Registry.get_default()
        self.assertEquals(registry1, registry2)

    def test_add_directory(self):
        pass

    def test_load_existing(self):
        # skipping since this is likely to have been loaded by another test
        return None

        if not self.registry.load(self.EXISTING_LIBRARY_PATH):
            self.fail()

    def test_load_unexisting(self):
        self.assertRaises(glib.GError, self.registry.load, self.NONEXISTING_LIBRARY_PATH)

    def test_load_invalid(self):
        self.assertRaises(glib.GError, self.registry.load, self.INVALID_LIBRARY_PATH)

    def test_load_directory_nonexisting(self):
        self.assertRaises(glib.GError, self.registry.load_directory, '')

    def test_load_directory_existing(self):
        if not self.registry.load_directory(os.getcwd()):
            self.fail()

    def test_unload(self):
        pass

    def test_load_all(self):
        self.assertTrue(self.registry.load_all())

    def test_register_source(self):
        pass

    def test_unregister_source(self):
        self.registry.load_all()
        sources = self.registry.get_sources(False)
        for source in sources:
            self.registry.unregister_source(source)
        sources = self.registry.get_sources(False)
        self.assertEqual(len(sources), 0)

    def test_lookup_source(self):
        self.registry.load_all()
        sources = self.registry.get_sources(False)

        if sources:
            expected = sources[0]
            search_id = expected.get_id()
            found = self.registry.lookup_source(search_id)

            self.assertEqual(expected, found)

    def test_lookup_source_nonexisting(self):
        self.registry.load_all()
        found = self.registry.lookup_source(self.NONEXISTING_SOURCE)
        self.assertFalse(found)

    def test_get_sources(self):
        sources = self.registry.get_sources(False)
        self.assertEqual(len(sources), 0)

        self.registry.load_all()
        sources = self.registry.get_sources(False)
        self.assertNotEqual(len(sources), 0)

    def test_get_sources_ordering(self):
        self.registry.load_all()
        ordered_sources = self.registry.get_sources(True)
        unordered_sources = self.registry.get_sources(False)

        unordered_sources.sort(key=Grl.MediaPlugin.get_rank)
        self.assertEquals(ordered_sources, unordered_sources)

    def test_get_sources_by_operations_supported_ops(self):
        self.registry.load_all()
        for op in constants.SUPPORTED_OPS:
            if op != Grl.SupportedOps.NONE:
                sources = self.registry.get_sources_by_operations(op, False)
                if sources:
                    supported_operations = sources[0].supported_operations()
                    self.assertTrue(supported_operations & op)

    def test_register_metadata_key(self):
        pass

    def test_lookup_metadata_key_existing_key(self):
        existing_key = self.registry.lookup_metadata_key(constants.KEY_ID)
        self.assertTrue(existing_key)

    def test_lookup_metadata_key_nonexisting_key(self):
        nonexisting_key = self.registry.lookup_metadata_key(constants.KEY_NONEXISTING)
        self.assertTrue(nonexisting_key == Grl.METADATA_KEY_INVALID)

    def test_lookup_metadata_key_singleton(self):
        a_key = self.registry.lookup_metadata_key(constants.KEY_ID)
        another_key = self.registry.lookup_metadata_key(constants.KEY_ID)
        self.assertEquals(a_key, another_key)

    def test_get_metadata_keys_contains(self):
        registered_key = self.registry.lookup_metadata_key(constants.KEY_ARTIST)
        metadata_keys = self.registry.get_metadata_keys()
        self.assertTrue(registered_key in metadata_keys)

    def test_get_metadata_keys_all_keys(self):
        registered_keys = []
        for k in constants.REGISTERED_KEYS:
            registered_keys.append(self.registry.lookup_metadata_key(k))

        metadata_keys = self.registry.get_metadata_keys()
        self.assertEquals(registered_keys.sort(), metadata_keys.sort())

    def test_add_config(self):
        pass
